@@ -111,15 +111,17 @@ static void collect_argv(Blob *pExtra, int iStart){
 **                present, are passed along verbatim.  The --force and
 **                --randomize options are not supported.
 **
+**    remote      Show remote hosts for all repositories.
+**
 **    repack      Look for extra compression in all repositories.
 **
 **    sync        Run a "sync" on all repositories.  Only the --verbose
 **                and --unversioned and --share-links options are supported.
 **
-**    set         Run the "setting" or "set" commands on all
-**                repositories.  These command are particularly useful in
-**                conjunction with the "max-loadavg" setting which cannot
-**                otherwise be set globally.
+**    set         Run the "setting" or "set" commands on all repositories.
+**                This command is useful for settings like "max-loadavg" which
+**                you usually want to be the same across all repositories
+**                on a server.
 **
 **    unset       Run the "unset" command on all repositories
 **
@@ -131,6 +133,9 @@ static void collect_argv(Blob *pExtra, int iStart){
 **                the --localauth option and automatically launch a
 **                web-browser
 **
+**    whatis      Run the "whatis" command on all repositories.  Only
+**                show output for repositories that have a match.           
+**
 **
 ** In addition, the following maintenance operations are supported:
 **
@@ -408,11 +413,15 @@ void all_cmd(void){
     zCmd = "cache -R";
     showLabel = 1;
     collect_argv(&extra, 3);
+  }else if( fossil_strcmp(zCmd, "whatis")==0 ){
+    zCmd = "whatis -q -R";
+    quiet = 1;
+    collect_argv(&extra, 3);
   }else{
     fossil_fatal("\"all\" subcommand should be one of: "
       "add cache changes clean dbstat extras fts-config git ignore "
       "info list ls pull push rebuild remote "
-      "server setting sync ui unset");
+      "server setting sync ui unset whatis");
   }
   verify_all_options();
   db_multi_exec("CREATE TEMP TABLE repolist(name,tag);");
