@@ -83,6 +83,8 @@ static const struct {
                              "merges\n" },
   { "display",    "chart",   "Command to run after \"next\".  \"chart\", "
                              "\"log\", \"status\", or \"none\"" },
+  { "linear",     "off",     "Do a linear scan rather than a true bisect, "
+                             "stopping at the first \"bad\" result"},
 };
 
 /*
@@ -202,7 +204,7 @@ static void bisect_append_skip(int rid){
 ** Create a TEMP table named "bilog" that contains the complete history
 ** of the current bisect.
 **
-** If iCurrent>0 then it is the RID of the current checkout and is included
+** If iCurrent>0 then it is the RID of the current check-out and is included
 ** in the history table.
 **
 ** If zDesc is not NULL, then it is the bid= query parameter to /timeline
@@ -240,6 +242,7 @@ int bisect_create_bilog_table(int iCurrent, const char *zDesc, int bDetail){
       if( rid==0 ) break;
       blob_appendf(&log, "%d", rid);
       zDesc += i;
+      while( zDesc[0]=='-' ) zDesc++;
     }
   }else{
     zLog = db_lget("bisect-log","");
@@ -329,6 +332,7 @@ char *bisect_permalink(void){
       }
     }
     zUuid = db_text(0,"SELECT lower(uuid) FROM blob WHERE rid=%d", rid);
+    if( blob_size(&link)>0 ) blob_append(&link, "-", 1);
     blob_appendf(&link, "%c%.10s", cPrefix, zUuid);
   }
   zResult = mprintf("%s", blob_str(&link));
@@ -375,7 +379,8 @@ static void bisect_chart(int sortByCkinTime){
 void bisect_reset(void){
   db_multi_exec(
     "DELETE FROM vvar WHERE name IN "
-    " ('bisect-good', 'bisect-bad', 'bisect-log', 'bisect-complete')"
+    " ('bisect-good', 'bisect-bad', 'bisect-log', 'bisect-complete',"
+    "  'bisect-linear')"
   );
 }
 
@@ -383,8 +388,8 @@ void bisect_reset(void){
 ** fossil bisect run [OPTIONS] COMMAND
 **
 ** Invoke COMMAND (with arguments) repeatedly to perform the bisect.
-** Options:
 **
+** Options:
 **    -i|--interactive          Prompt user for decisions rather than
 **                              using the return code from COMMAND
 */
@@ -466,17 +471,17 @@ static void bisect_run(void){
 ** Usage: %fossil bisect SUBCOMMAND ...
 **
 ** Run various subcommands useful for searching back through the change
-** history for a particular checkin that causes or fixes a problem.
+** history for a particular check-in that causes or fixes a problem.
 **
 ** > fossil bisect bad ?VERSION?
 **
 **       Identify version VERSION as non-working.  If VERSION is omitted,
-**       the current checkout is marked as non-working.
+**       the current check-out is marked as non-working.
 **
 ** > fossil bisect good ?VERSION?
 **
 **       Identify version VERSION as working.  If VERSION is omitted,
-**       the current checkout is marked as working.
+**       the current check-out is marked as working.
 **
 ** > fossil bisect log
 ** > fossil bisect chart
@@ -504,15 +509,16 @@ static void bisect_run(void){
 **
 **       Invoke COMMAND repeatedly to run the bisect.  The exit code for
 **       COMMAND should be 0 for "good", 125 for "skip", and any other value
-**       for "bad".  Options:
+**       for "bad".
 **
+**       Options:
 **          -i|--interactive    Prompt the user for the good/bad/skip decision
 **                              after each step, rather than using the exit
 **                              code from COMMAND
 **
 ** > fossil bisect skip ?VERSION?
 **
-**       Cause VERSION (or the current checkout if VERSION is omitted) to
+**       Cause VERSION (or the current check-out if VERSION is omitted) to
 **       be ignored for the purpose of the current bisect.  This might
 **       be done, for example, because VERSION does not compile correctly
 **       or is otherwise unsuitable to participate in this bisect.
@@ -630,7 +636,12 @@ void bisect_cmd(void){
     char *zDisplay = db_lget("bisect-display","chart");
     int m = (int)strlen(zDisplay);
     bisect_path();
-    pMid = path_midpoint();
+    if( db_lget_boolean("bisect-linear",0) ){
+      pMid = path_next();
+      if( pMid && pMid->rid==db_lget_int("checkout",0) ) pMid = 0;
+    }else{
+      pMid = path_midpoint();
+    }
     if( pMid==0 ){
       fossil_print("bisect complete\n");
       db_lset_int("bisect-complete",1);
