@@ -316,13 +316,11 @@ void cgi_set_cookie(
   }
   if( lifetime!=0 ){
     blob_appendf(&extraHeader,
-       "Set-Cookie: %s=%t; Path=%s; max-age=%d; HttpOnly; "
-       "%s Version=1\r\n",
+       "Set-Cookie: %s=%t; Path=%s; max-age=%d; HttpOnly; %s\r\n",
        zName, lifetime>0 ? zValue : "null", zPath, lifetime, zSecure);
   }else{
     blob_appendf(&extraHeader,
-       "Set-Cookie: %s=%t; Path=%s; HttpOnly; "
-       "%s Version=1\r\n",
+       "Set-Cookie: %s=%t; Path=%s; HttpOnly; %s\r\n",
        zName, zValue, zPath, zSecure);
   }
 }
@@ -499,7 +497,13 @@ void cgi_reply(void){
     assert( rangeEnd==0 );
     blob_appendf(&hdr, "Status: %d %s\r\n", iReplyStatus, zReplyStatus);
   }
-  if( etag_tag()[0]!=0 ){
+  if( etag_tag()[0]!=0
+   && iReplyStatus==200
+   && strcmp(zContentType,"text/html")!=0
+  ){
+    /* Do not cache HTML replies as those will have been generated and
+    ** will likely, therefore, contains a nonce and we want that nonce to
+    ** be different every time. */
     blob_appendf(&hdr, "ETag: %s\r\n", etag_tag());
     blob_appendf(&hdr, "Cache-Control: max-age=%d\r\n", etag_maxage());
     if( etag_mtime()>0 ){
@@ -520,7 +524,7 @@ void cgi_reply(void){
 
   /* Add headers to turn on useful security options in browsers. */
   blob_appendf(&hdr, "X-Frame-Options: SAMEORIGIN\r\n");
-  /* This stops fossil pages appearing in frames or iframes, preventing
+  /* The previous stops fossil pages appearing in frames or iframes, preventing
   ** click-jacking attacks on supporting browsers.
   **
   ** Other good headers would be
@@ -536,9 +540,6 @@ void cgi_reply(void){
   ** a CGI script.
   */
 
-  /* Content intended for logged in users should only be cached in
-  ** the browser, not some shared location.
-  */
   if( iReplyStatus!=304 ) {
     blob_appendf(&hdr, "Content-Type: %s%s\r\n", zContentType,
                  content_type_charset(zContentType));
@@ -701,20 +702,61 @@ int cgi_same_origin(void){
 }
 
 /*
-** Return true if the current request appears to be safe from a
-** Cross-Site Request Forgery (CSRF) attack.  Conditions that must
-** be met:
-**
-**    *   The HTTP_REFERER must have the same origin
-**    *   The REQUEST_METHOD must be POST - or requirePost==0
+** Return true if the current CGI request is a POST request
 */
-int cgi_csrf_safe(int requirePost){
-  if( requirePost ){
-    const char *zMethod = P("REQUEST_METHOD");
-    if( zMethod==0 ) return 0;
-    if( strcmp(zMethod,"POST")!=0 ) return 0;
+static int cgi_is_post_request(void){
+  const char *zMethod = P("REQUEST_METHOD");
+  if( zMethod==0 ) return 0;
+  if( strcmp(zMethod,"POST")!=0 ) return 0;
+  return  1;
+}
+
+/*
+** Return true if the current request appears to be safe from a
+** Cross-Site Request Forgery (CSRF) attack.  The level of checking
+** is determined by the parameter.  The higher the number, the more
+** secure we are:
+**
+**    0:     Request must come from the same origin
+**    1:     Same origin and must be a POST request
+**    2:     All of the above plus must have a valid CSRF token
+**
+** Results are cached in the g.okCsrf variable.  The g.okCsrf value
+** has meaning as follows:
+**
+**    -1:   Not a secure request
+**     0:   Status unknown
+**     1:   Request comes from the same origin
+**     2:   (1) plus it is a POST request
+**     3:   (2) plus there is a valid "csrf" token in the request
+*/
+int cgi_csrf_safe(int securityLevel){
+  if( g.okCsrf<0 ) return 0;
+  if( g.okCsrf==0 ){
+    if( !cgi_same_origin() ){
+      g.okCsrf = -1;
+    }else{
+      g.okCsrf = 1;
+      if( cgi_is_post_request() ){
+        g.okCsrf = 2;
+        if( fossil_strcmp(P("csrf"), g.zCsrfToken)==0 ){
+          g.okCsrf = 3;
+        }
+      }
+    }
+  }
+  return g.okCsrf >= (securityLevel+1);
+}
+
+/*
+** Verify that CSRF defenses are maximal - that the request comes from
+** the same origin, that it is a POST request, and that there is a valid
+** "csrf" token.  If this is not the case, fail immediately.
+*/
+void cgi_csrf_verify(void){
+  if( !cgi_csrf_safe(2) ){
+    fossil_fatal("Cross-site Request Forgery detected");
   }
-  return cgi_same_origin();
 }
 
 /*
@@ -731,6 +773,7 @@ static struct QParam {   /* One entry for each query parameter or cookie */
   int seq;                  /* Order of insertion */
   char isQP;                /* True for query parameters */
   char cTag;                /* Tag on query parameters */
+  char isFetched;           /* 1 if the var is requested via P/PD() */
 } *aParamQP;             /* An array of all parameters and cookies */
 
 /*
@@ -758,6 +801,7 @@ void cgi_set_parameter_nocopy(const char *zName, const char *zValue, int isQP){
   aParamQP[nUsedQP].seq = seqQP++;
   aParamQP[nUsedQP].isQP = isQP;
   aParamQP[nUsedQP].cTag = 0;
+  aParamQP[nUsedQP].isFetched = 0;
   nUsedQP++;
   sortQP = 1;
 }
@@ -1485,6 +1529,7 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
     c = fossil_strcmp(aParamQP[mid].zName, zName);
     if( c==0 ){
       CGIDEBUG(("mem-match [%s] = [%s]\n", zName, aParamQP[mid].zValue));
+      aParamQP[mid].isFetched = 1;
       return aParamQP[mid].zValue;
     }else if( c>0 ){
       hi = mid-1;
@@ -1512,20 +1557,23 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
 /*
 ** Renders the "begone, spider" page and exits.
 */
-static void cgi_begone_spider(void){
+static void cgi_begone_spider(const char *zName){
   Blob content = empty_blob;
-
   cgi_set_content(&content);
   style_set_current_feature("test");
+  style_submenu_enable(0);
   style_header("Malicious Query Detected");
-  @ <h2>Begone, Fiend!</h2>
-  @ <p>This page was generated because Fossil believes it has
-  @ detected an SQL injection attack. If you believe you are seeing
-  @ this in error, contact the developers on the Fossil-SCM Forum.  Type
+  @ <h2>Begone, Knave!</h2>
+  @ <p>This page was generated because Fossil detected an (unsuccessful)
+  @ SQL injection attack or other nefarious content in your HTTP request.
+  @
+  @ <p>If you believe you are innocent and have reached this page in error,
+  @ contact the Fossil developers on the Fossil-SCM Forum.  Type
   @ "fossil-scm forum" into any search engine to locate the Fossil-SCM Forum.
   style_finish_page();
-  cgi_set_status(404,"Robot Attack Detected");
+  cgi_set_status(418,"I'm a teapot");
   cgi_reply();
+  fossil_errorlog("Xpossible hack attempt - 418 response on \"%s\"", zName);
   exit(0);
 }
 
@@ -1545,9 +1593,9 @@ static void cgi_begone_spider(void){
 ** words, this is an effort to reduce the CPU load imposed by malicious
 ** spiders.  It is not an effect defense against SQL injection vulnerabilities.
 */
-void cgi_value_spider_check(const char *zTxt){
+void cgi_value_spider_check(const char *zTxt, const char *zName){
   if( g.zLogin==0 && looks_like_sql_injection(zTxt) ){
-    cgi_begone_spider();
+    cgi_begone_spider(zName);
   }
 }
 
@@ -1560,7 +1608,7 @@ const char *cgi_parameter_nosql(const char *zName, const char *zDefault){
   const char *zTxt = cgi_parameter(zName, zDefault);
 
   if( zTxt!=zDefault ){
-    cgi_value_spider_check(zTxt);
+    cgi_value_spider_check(zTxt, zName);
   }
   return zTxt;
 }
@@ -1735,29 +1783,41 @@ void cgi_load_environment(void){
 ** The eDest parameter determines where the output is shown:
 **
 **     eDest==0:    Rendering as HTML into the CGI reply
-**     eDest==1:    Written to stderr
+**     eDest==1:    Written to fossil_trace
 **     eDest==2:    Written to cgi_debug
+**     eDest==3:    Written to out  (Used only by fossil_errorlog())
 */
-void cgi_print_all(int showAll, unsigned int eDest){
+void cgi_print_all(int showAll, unsigned int eDest, FILE *out){
   int i;
   cgi_parameter("","");  /* Force the parameters into sorted order */
   for(i=0; i<nUsedQP; i++){
     const char *zName = aParamQP[i].zName;
-    if( !showAll ){
-      if( fossil_stricmp("HTTP_COOKIE",zName)==0 ) continue;
-      if( fossil_strnicmp("fossil-",zName,7)==0 ) continue;
+    const char *zValue = aParamQP[i].zValue;
+    if( fossil_stricmp("HTTP_COOKIE",zName)==0
+     || fossil_strnicmp("fossil-",zName,7)==0
+    ){
+      if( !showAll ) continue;
+      if( eDest==3 ) zValue = "...";
     }
     switch( eDest ){
       case 0: {
-        cgi_printf("%h = %h  <br>\n", zName, aParamQP[i].zValue);
+        cgi_printf("%h = %h  <br>\n", zName, zValue);
         break;
       }
-      case 1: {  
-        fossil_trace("%s = %s\n", zName, aParamQP[i].zValue);
+      case 1: {
+        fossil_trace("%s = %s\n", zName, zValue);
         break;
       }
       case 2: {
-        cgi_debug("%s = %s\n", zName, aParamQP[i].zValue);
+        cgi_debug("%s = %s\n", zName, zValue);
+        break;
+      }
+      case 3: {
+        if( strlen(zValue)>100 ){
+          fprintf(out,"%s = %.100s...\n", zName, zValue);
+        }else{
+          fprintf(out,"%s = %s\n", zName, zValue);
+        }
         break;
       }
     }
@@ -2686,3 +2746,41 @@ int cgi_from_mobile(void){
   if( sqlite3_strglob("*iPad*", zAgent)==0 ) return 0;
   return sqlite3_strlike("%mobile%", zAgent, 0)==0;
 }
+
+/*
+** Look for query or POST parameters that:
+**
+**    (1)  Have not been used
+**    (2)  Appear to be malicious attempts to break into or otherwise
+**         harm the system, for example via SQL injection
+**
+** If any such parameters are seen, a 418 ("I'm a teapot") return is
+** generated and processing aborts - this routine does not return.
+**
+** When Fossil is launched via CGI from althttpd, the 418 return signals
+** the webserver to put the requestor IP address into "timeout", blocking
+** subsequent requests for 5 minutes.
+**
+** Fossil is not subject to any SQL injections, as far as anybody knows.
+** This routine is not necessary for the security of the system (though
+** an extra layer of security never hurts).  The main purpose here is
+** to shutdown malicious attack spiders and prevent them from burning
+** lots of CPU cycles and bogging down the website.  In other words, the
+** objective of this routine is to help prevent denial-of-service.
+**
+** Usage Hint: Put a call to this routine as late in the webpage
+** implementation as possible, ideally just before it begins doing
+** potentially CPU-intensive computations and after all query parameters
+** have been consulted.
+*/
+void cgi_check_for_malice(void){
+  struct QParam * pParam;
+  int i;
+  for(i = 0; i < nUsedQP; ++i){
+    pParam = &aParamQP[i];
+    if(0 == pParam->isFetched
+       && fossil_islower(pParam->zName[0])){
+      cgi_value_spider_check(pParam->zValue, pParam->zName);
+    }
+  }
+}
