@@ -112,6 +112,33 @@ static int file_dir_match(FileDirList *p, const char *zFile){
   return 0;
 }
 
+/*
+** Print details about the compared versions - possibly the working directory
+** or the undo buffer.  For check-ins, show hash and commit time.
+** 
+** This is intended primarily to go into the "header garbage" that is ignored
+** by patch(1).
+**
+** zFrom and zTo are interpreted as symbolic version names, unless they
+** start with '(', in which case they are printed directly.
+*/
+void diff_print_versions(const char *zFrom, const char *zTo, DiffConfig *pCfg){
+  if( (pCfg->diffFlags & (DIFF_SIDEBYSIDE|DIFF_BRIEF|DIFF_NUMSTAT|
+        DIFF_HTML|DIFF_WEBPAGE|DIFF_BROWSER|DIFF_JSON|DIFF_TCL))==0 ){
+    fossil_print("Fossil-Diff-From:  %s\n",
+      zFrom[0]=='(' ? zFrom : mprintf("%S %s",
+        rid_to_uuid(symbolic_name_to_rid(zFrom, "ci")),
+        db_text("","SELECT datetime(%f)||' UTC'",
+          symbolic_name_to_mtime(zFrom, 0))));
+    fossil_print("Fossil-Diff-To:    %s\n",
+      zTo[0]=='(' ? zTo : mprintf("%S %s",
+        rid_to_uuid(symbolic_name_to_rid(zTo, "ci")),
+        db_text("","SELECT datetime(%f)||' UTC'",
+          symbolic_name_to_mtime(zTo, 0))));
+    fossil_print("%.66c\n", '-');
+  }
+}
+
 /*
 ** Print the "Index:" message that patches wants to see at the top of a diff.
 */
@@ -127,6 +154,7 @@ void diff_print_index(const char *zFile, DiffConfig *pCfg, Blob *pOut){
 /*
 ** Print the +++/--- filename lines or whatever filename information
 ** is appropriate for the output format.
+**
 */
 void diff_print_filenames(
   const char *zLeft,      /* Name of the left file */
@@ -342,7 +370,7 @@ void diff_begin(DiffConfig *pCfg){
     tempDiffFilename = sqlite3_mprintf("%z.html", tempDiffFilename);
     diffOut = fossil_freopen(tempDiffFilename,"wb",stdout);
     if( diffOut==0 ){
-      fossil_fatal("unable to create temporary file \"%s\"", 
+      fossil_fatal("unable to create temporary file \"%s\"",
                    tempDiffFilename);
     }
 #ifndef _WIN32
@@ -662,6 +690,10 @@ void diff_against_disk(
       vid
     );
   }
+  if( (pCfg->diffFlags & DIFF_SHOW_VERS)!=0 ){
+    diff_print_versions(zFrom ? zFrom : db_lget("checkout-hash", 0),
+      "(workdir)", pCfg);
+  }
   db_prepare(&q, "%s", blob_sql_text(&sql));
   blob_reset(&sql);
   while( db_step(&q)==SQLITE_ROW ){
@@ -746,6 +778,9 @@ static void diff_against_undo(
   Blob content;
   db_prepare(&q, "SELECT pathname, content FROM undo");
   blob_init(&content, 0, 0);
+  if( (pCfg->diffFlags & DIFF_SHOW_VERS)!=0 ){
+    diff_print_versions("(undo)", "(workdir)", pCfg);
+  }  
   while( db_step(&q)==SQLITE_ROW ){
     char *zFullName;
     const char *zFile = (const char*)db_column_text(&q, 0);
@@ -830,7 +865,9 @@ static void diff_two_versions(
   pTo = manifest_get_by_name(zTo, 0);
   manifest_file_rewind(pTo);
   pToFile = manifest_file_next(pTo,0);
-
+  if( (pCfg->diffFlags & DIFF_SHOW_VERS)!=0 ){
+    diff_print_versions(zFrom, zTo, pCfg);
+  }    
   while( pFromFile || pToFile ){
     int cmp;
     if( pFromFile==0 ){
@@ -992,7 +1029,7 @@ const char *diff_get_binary_glob(void){
 ** Usage: %fossil diff|gdiff ?OPTIONS? ?FILE1? ?FILE2 ...?
 **
 ** Show the difference between the current version of each of the FILEs
-** specified (as they exist on disk) and that same file as it was checked
+** specified (as they exist on disk) and that same file as it was checked-
 ** out.  Or if the FILE arguments are omitted, show all unsaved changes
 ** currently in the working check-out.
 **
@@ -1007,7 +1044,7 @@ const char *diff_get_binary_glob(void){
 **    --webpage         Format output as HTML
 **    --webpage -y      HTML output in the side-by-side format
 **
-** The "--from VERSION" option is used it specifies the source check-in
+** The "--from VERSION" option is used to specify the source check-in
 ** for the diff operation.  If not specified, the source check-in is the
 ** base check-in for the current check-out. Similarly, the "--to VERSION"
 ** option specifies the check-in from which the second version of the file
@@ -1016,7 +1053,7 @@ const char *diff_get_binary_glob(void){
 ** shows the changes made by check-in VERSION relative to its primary parent.
 ** The "--branch BRANCHNAME" shows all the changes on the branch BRANCHNAME.
 **
-** The "-i" command-line option forces the use of Fossils own the internal
+** The "-i" command-line option forces the use of Fossil's own internal
 ** diff logic rather than any external diff program that might be configured
 ** using the "setting" command.  If no external diff program is configured,
 ** then the "-i" option is a no-op.  The "-i" option converts "gdiff" into
@@ -1026,8 +1063,8 @@ const char *diff_get_binary_glob(void){
 ** when using an external diff program.
 **
 ** The "--binary" option causes files matching the glob PATTERN to be treated
-** as binary when considering if they should be used with external diff program.
-** This option overrides the "binary-glob" setting.
+** as binary when considering if they should be used with the external diff
+** program.  This option overrides the "binary-glob" setting.
 **
 ** These command show differences between managed files. Use the "fossil xdiff"
 ** command to see differences in unmanaged files.
@@ -1041,7 +1078,8 @@ const char *diff_get_binary_glob(void){
 **   --by                        Shorthand for "--browser -y"
 **   -ci|--checkin VERSION       Show diff of all changes in VERSION
 **   --command PROG              External diff program. Overrides "diff-command"
-**   -c|--context N              Show N lines of context around each change
+**   -c|--context N              Show N lines of context around each change, with
+**                               negative N meaning show all content
 **   --diff-binary BOOL          Include binary files with external commands
 **   --exec-abs-paths            Force absolute path names on external commands
 **   --exec-rel-paths            Force relative path names on external commands
@@ -1050,7 +1088,7 @@ const char *diff_get_binary_glob(void){
 **   -i|--internal               Use internal diff logic
 **   --json                      Output formatted as JSON
 **   -N|--new-file               Alias for --verbose
-**   --numstat                   Show only the number of lines delete and added
+**   --numstat                   Show only the number of added and deleted lines
 **   -y|--side-by-side           Side-by-side diff
 **   --strip-trailing-cr         Strip trailing CR
 **   --tcl                       TCL-formated output used internally by --tk
@@ -1060,6 +1098,7 @@ const char *diff_get_binary_glob(void){
 **   --undo                      Diff against the "undo" buffer
 **   --unified                   Unified diff
 **   -v|--verbose                Output complete text of added or deleted files
+**   -h|--versions               Show compared versions in the diff header
 **   --webpage                   Format output as a stand-alone HTML webpage
 **   -W|--width N                Width of lines in side-by-side diff
 **   -Z|--ignore-trailing-space  Ignore changes to end-of-line whitespace
@@ -1099,10 +1138,14 @@ void diff_cmd(void){
     fossil_fatal("cannot use --checkin together with --from or --to");
   }
   g.diffCnt[0] = g.diffCnt[1] = g.diffCnt[2] = 0;
-  if( zTo==0 || againstUndo ){
-    db_must_be_within_tree();
-  }else if( zFrom==0 ){
-    fossil_fatal("must use --from if --to is present");
+  if( 0==zCheckin ){
+    if( zTo==0 || againstUndo ){
+      db_must_be_within_tree();
+    }else if( zFrom==0 ){
+      fossil_fatal("must use --from if --to is present");
+    }else{
+      db_find_and_open_repository(0, 0);
+    }
   }else{
     db_find_and_open_repository(0, 0);
   }
@@ -1180,10 +1223,12 @@ void vpatch_page(void){
   const char *zFrom = P("from");
   const char *zTo = P("to");
   DiffConfig DCfg;
+  cgi_check_for_malice();
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
   if( zFrom==0 || zTo==0 ) fossil_redirect_home();
 
+  fossil_nice_default();
   cgi_set_content_type("text/plain");
   diff_config_init(&DCfg, DIFF_VERBOSE);
   diff_two_versions(zFrom, zTo, &DCfg, 0);
