@@ -274,8 +274,9 @@ void finfo_cmd(void){
 ** Other versions may be specified using the -r option.
 **
 ** Options:
-**    -R|--repository REPO       Extract artifacts from repository REPO
-**    -r VERSION                 The specific check-in containing the file
+**    -o|--out OUTFILE         For exactly one given FILENAME, write to OUTFILE
+**    -R|--repository REPO     Extract artifacts from repository REPO
+**    -r VERSION               The specific check-in containing the file
 **
 ** See also: [[finfo]]
 */
@@ -283,17 +284,27 @@ void cat_cmd(void){
   int i;
   Blob content, fname;
   const char *zRev;
+  const char *zFileName;
   db_find_and_open_repository(0, 0);
   zRev = find_option("r","r",1);
+  zFileName = find_option("out","o",1);
 
   /* We should be done with options.. */
   verify_all_options();
 
+  if ( zFileName && g.argc>3 ){
+    fossil_fatal("output file can only be given when retrieving a single file");
+  }
+
   for(i=2; i<g.argc; i++){
     file_tree_name(g.argv[i], &fname, 0, 1);
     blob_zero(&content);
     historical_blob(zRev, blob_str(&fname), &content, 1);
-    blob_write_to_file(&content, "-");
+    if ( g.argc==3 && zFileName ){
+      blob_write_to_file(&content, zFileName);
+    }else{
+      blob_write_to_file(&content, "-");
+    }
     blob_reset(&fname);
     blob_reset(&content);
   }
@@ -414,6 +425,7 @@ void finfo_page(void){
     }
   }
   url_add_parameter(&url, "name", zFilename);
+  cgi_check_for_malice();
   blob_zero(&sql);
   if( ridCi ){
     /* If we will be tracking changes across renames, some extra temp
@@ -758,7 +770,7 @@ void finfo_page(void){
     if( fDebug & FINFO_DEBUG_MLINK ){
       int ii;
       char *zAncLink;
-      @ <br />fid=%d(frid) \
+      @ <br>fid=%d(frid) \
       @ graph-id=%lld(frid>0?(GraphRowId)frid*(mxfnid+1)+fnid:fpid+1000000000) \
       @ pid=%d(fpid) mid=%d(fmid) fnid=%d(fnid) \
       @ pfnid=%d(pfnid) mxfnid=%d(mxfnid)
@@ -921,7 +933,7 @@ void mlink_page(void){
     @ <h1>MLINK table for check-in %h(zCI)</h1>
     render_checkin_context(mid, 0, 1, 0);
     style_table_sorter();
-    @ <hr />
+    @ <hr>
     @ <div class='brlist'>
     @ <table class='sortable' data-column-types='ttxtttt' data-init-sort='1'>
     @ <thead><tr>
