@@ -384,7 +384,7 @@ window.fossil.onPageLoad(function(){
             if(ev.detail.key===setting) f(ev.detail);
           }, false);
         },
-        /* Default values of settings. These are used for intializing
+        /* Default values of settings. These are used for initializing
            the setting event listeners and config view UI. */
         defaults:{
           /* When on, inbound images are displayed inlined, else as a
@@ -704,6 +704,11 @@ window.fossil.onPageLoad(function(){
       else if(e.$isToggling) return;
       e.$isToggling = true;
       const content = e.querySelector('.content-target');
+      if(!content){
+        console.warn("Should not be possible: trying to toggle text",
+                     "mode of a message with no .content-target.", e);
+        return;
+      }
       if(!content.$elems){
         content.$elems = [
           content.firstElementChild, // parsed elem
@@ -716,8 +721,25 @@ window.fossil.onPageLoad(function(){
             ? content.$elems[1]
             : content.$elems[0]
         );
+        D.clearElement(content);
+        if(child===content.$elems[1]){
+          /* When showing the unformatted version, inject a
+             copy-to-clipboard button. This is a workaround for
+             mouse-copying from that field collecting twice as many
+             newlines as it should (for unknown reasons). */
+          const cpId = 'copy-to-clipboard-'+id;
+          /* ^^^ copy button element ID, needed for LABEL element
+             pairing.  Recall that we destroy all child elements of
+             `content` each time we hit this block, so we can reuse
+             that element ID on subsequent toggles. */
+          const btnCp = D.attr(D.addClass(D.span(),'copy-button'), 'id', cpId);
+          F.copyButton(btnCp, {extractText: ()=>child._xmsgRaw});
+          const lblCp = D.label(cpId, "Copy unformatted text");
+          lblCp.addEventListener('click',()=>btnCp.click(), false);
+          D.append(content, D.append(D.addClass(D.span(), 'nobr'), btnCp, lblCp));
+        }
         delete e.$isToggling;
-        D.append(D.clearElement(content), child);
+        D.append(content, child);
         return;
       }
       // We need to fetch the plain-text version...
@@ -727,6 +749,7 @@ window.fossil.onPageLoad(function(){
         responseType: 'json',
         onload: function(msg){
           content.$elems[1] = D.append(D.pre(),msg.xmsg);
+          content.$elems[1]._xmsgRaw = msg.xmsg/*used for copy-to-clipboard feature*/;
           self.toggleTextMode(e);
         },
         aftersend:function(){
@@ -893,19 +916,54 @@ window.fossil.onPageLoad(function(){
       ].join('');
     };
 
+    /**
+       Returns true if this page believes it can embed a view of the
+       file wrapped by the given message object, else returns false.
+    */
     const canEmbedFile = function f(msg){
       if(!f.$rx){
-        f.$rx = /\.((html?)|(txt))$/i;
+        f.$rx = /\.((html?)|(txt)|(md)|(wiki)|(pikchr))$/i;
         f.$specificTypes = [
           'text/plain',
-          'text/html'
+          'text/html',
+          'text/x-markdown',
+          /* Firefox sends text/markdown when uploading .md files */
+          'text/markdown',
+          'text/x-pikchr',
+          'text/x-fossil-wiki'
           // add more as we discover which ones Firefox won't
           // force the user to try to download.
         ];
       }
       if(msg.fmime){
-        return (msg.fmime.startsWith("image/")
-                || f.$specificTypes.indexOf(msg.fmime)>=0);
+        if(msg.fmime.startsWith("image/")
+           || f.$specificTypes.indexOf(msg.fmime)>=0){
+          return true;
+        }
+      }
+      return (msg.fname && f.$rx.test(msg.fname));
+    };
+
+    /**
+      Returns true if the given message object "should"
+      be embedded in fossil-rendered form instead of
+      raw content form. This is only intended to be passed
+      message objects for which canEmbedFile() returns true.
+    */
+    const shouldWikiRenderEmbed = function f(msg){
+      if(!f.$rx){
+        f.$rx = /\.((md)|(wiki)|(pikchr))$/i;
+        f.$specificTypes = [
+          'text/x-markdown',
+          'text/markdown' /* Firefox-uploaded md files */,
+          'text/x-pikchr',
+          'text/x-fossil-wiki'
+          // add more as we discover which ones Firefox won't
+          // force the user to try to download.
+        ];
+      }
+      if(msg.fmime){
+        if(f.$specificTypes.indexOf(msg.fmime)>=0) return true;
       }
       return msg.fname && f.$rx.test(msg.fname);
     };
@@ -993,12 +1051,15 @@ window.fossil.onPageLoad(function(){
             if(canEmbedFile(m)){
               /* Add an option to embed HTML attachments in an iframe. The primary
                  use case is attached diffs. */
+              const shouldWikiRender = shouldWikiRenderEmbed(m);
+              const downloadArgs = shouldWikiRender ? '?render' : '';
               D.addClass(contentTarget, 'wide');
               const embedTarget = this.e.content;
               const self = this;
               const btnEmbed = D.attr(D.checkbox("1", false), 'id',
                                       'embed-'+ds.msgid);
-              const btnLabel = D.label(btnEmbed, "Embed");
+              const btnLabel = D.label(btnEmbed, shouldWikiRender
+                                       ? "Embed (fossil-rendered)" : "Embed");
               /* Maintenance reminder: do not disable the toggle
                  button while the content is loading because that will
                  cause it to get stuck in disabled mode if the browser
@@ -1020,7 +1081,7 @@ window.fossil.onPageLoad(function(){
                   self.e.$iframeLoaded = true;
                   adjustIFrameSize(self);
                 });
-                iframe.setAttribute('src', downloadUri);
+                iframe.setAttribute('src', downloadUri + downloadArgs);
               });
               D.append(w, btnEmbed, btnLabel);
             }
@@ -1137,11 +1198,15 @@ window.fossil.onPageLoad(function(){
               ));
               const toolbar2 = D.addClass(D.div(), 'toolbar');
               D.append(this.e, toolbar2);
-              D.append(toolbar2, D.button(
-                "Toggle text mode", function(){
-                  self.hide();
-                  Chat.toggleTextMode(eMsg);
-                }));
+              if(eMsg.querySelector('.content-target')){
+                /* ^^^ messages with only an embedded image have no
+                   .content-target area. */
+                D.append(toolbar2, D.button(
+                  "Toggle text mode", function(){
+                    self.hide();
+                    Chat.toggleTextMode(eMsg);
+                  }));
+              }
               if(eMsg.dataset.xfrom){
                 /* Add a link to the /timeline filtered on this user. */
                 const timelineLink = D.attr(
@@ -1352,25 +1417,31 @@ window.fossil.onPageLoad(function(){
     if(!f.spaces){
       f.spaces = /\s+$/;
       f.markdownContinuation = /\\\s+$/;
+      f.spaces2 = /\s{3,}$/;
     }
     this.setCurrentView(this.e.viewMessages);
     const fd = new FormData();
     const fallback = {msg: this.inputValue()};
-    var msg = fallback.msg.trim();
+    var msg = fallback.msg;
     if(msg && (msg.indexOf('\n')>0 || f.spaces.test(msg))){
-      /* Cosmetic: trim whitespace from the ends of lines to try to
+      /* Cosmetic: trim most whitespace from the ends of lines to try to
          keep copy/paste from terminals, especially wide ones, from
          forcing a horizontal scrollbar on all clients. This breaks
          markdown's use of blackslash-space-space for paragraph
          continuation, but *not* doing this affects all clients every
          time someone pastes in console copy/paste from an affected
          platform. We seem to have narrowed to the console pasting
-         problem to users of tmux. Most consoles don't behave
-         that way. */
+         problem to users of tmux together with certain apps (vim, at
+         a minimum). Most consoles don't behave that way.
+
+         We retain two trailing spaces so that markdown conventions
+         which use end-of-line spacing aren't broken by this
+         stripping.
+      */
       const xmsg = msg.split('\n');
       xmsg.forEach(function(line,ndx){
         if(!f.markdownContinuation.test(line)){
-          xmsg[ndx] = line.trimRight();
+          xmsg[ndx] = line.replace(f.spaces2, '  ');
         }
       });
       msg = xmsg.join('\n');
