@@ -63,7 +63,7 @@
       outText: E('#pikchr-output-text'),
       pikOutWrapper: E('#pikchr-output-wrapper'),
       pikOut: E('#pikchr-output'),
-      btnRender: E('#btn-render')      
+      btnRender: E('#btn-render')
     },
     renderModes: ['svg'/*SVG must be at index 0*/,'markdown', 'wiki', 'text'],
     renderModeLabels: {
@@ -172,10 +172,10 @@
     PS.renderModeLabels[PS.renderModes[PS.renderModes.selectedIndex]];
 
   /**
-     The 'pikchrshow-ready' event is fired (with no payload) when the
+     The 'pikchr-ready' event is fired (with no payload) when the
      wasm module has finished loading. */
-  PS.addMsgHandler('pikchrshow-ready', function(){
-    PS.clearMsgHandlers('pikchrshow-ready');
+  PS.addMsgHandler('pikchr-ready', function(){
+    PS.clearMsgHandlers('pikchr-ready');
     F.page.onPikchrshowLoaded();
   });
 
@@ -679,6 +679,44 @@ text "HIGH" big bold "QUALITY" big bold at 0.9 heading  -120 from O  color blue
 text "EXPENSIVE" at 0.55 below O  color cyan
 text "SLOW" at 0.55 heading  -60 from O  color magenta
 text "POOR" "QUALITY" at 0.55 heading   60 from O  color gold
+`},{name:"Precision Arrows",code:`
+# Source: https://pikchr.org/home/forumpost/7f2f9a03eb
+define quiver {
+	dot invis at 0.5 < $1.ne , $1.e >
+	dot invis at 0.5 < $1.nw , $1.w >
+	dot invis at 0.5 < $1.se , $1.e >
+	dot invis at 0.5 < $1.sw , $1.w >
+
+	dot at $2 right of 4th previous dot
+        dot at $3 right of 4th previous dot
+	dot at $4 right of 4th previous dot
+        dot at $5 right of 4th previous dot
+	arrow <- from previous dot to 2nd previous dot
+	arrow -> from 3rd previous dot to 4th previous dot
+}
+
+define show_compass_l {
+	dot color red  at $1.e " .e" ljust
+	dot same at $1.ne " .ne" ljust above
+	line thick color green from previous to 2nd last dot
+}
+
+define show_compass_r {
+	dot color red  at $1.w " .w" ljust
+	dot same at $1.nw " .nw" ljust above
+	line thick color green from previous to 2nd last dot
+}
+
+PROGRAM: file "Program" rad 45px
+show_compass_l(PROGRAM)
+QUIVER: box invis ht 0.75
+DATABASE: oval "Database" ht 0.75 wid 1.1
+show_compass_r(DATABASE)
+
+quiver(QUIVER, 5px, -5px, 5px, 0px)
+
+text "Query" with .c at 0.1in above last arrow
+text "Records" with .c at 0.1in below 2nd last arrow
 `}
   ];
 
