@@ -298,12 +298,16 @@ static void finish_commit(void){
     if( zUuid==0 ) continue;
     blob_appendf(&record, "F %F %s", gg.aFile[i].zName, zUuid);
     if( gg.aFile[i].isExe ){
-      blob_append(&record, " x\n", 3);
+      blob_append(&record, " x", 2);
     }else if( gg.aFile[i].isLink ){
-      blob_append(&record, " l\n", 3);
+      blob_append(&record, " l", 2);
     }else{
-      blob_append(&record, "\n", 1);
+      blob_append(&record, " w", 2);
     }
+    if( gg.aFile[i].zPrior!=0 ){
+      blob_appendf(&record, " %F", gg.aFile[i].zPrior);
+    }
+    blob_append(&record, "\n", 1);
   }
   if( gg.zFrom ){
     blob_appendf(&record, "P %s", gg.zFrom);
@@ -544,7 +548,7 @@ static void dequote_git_filename(char *zName){
 
 static struct{
   const char *zMasterName;    /* Name of master branch */
-  int authorFlag;             /* Use author as checkin committer */
+  int authorFlag;             /* Use author as check-in committer */
   int nGitAttr;               /* Number of Git --attribute entries */
   struct {                    /* Git --attribute details */
     char *zUser;
@@ -599,7 +603,7 @@ static void git_fast_import(FILE *pIn){
       ** None of the above is explained in the git-fast-export
       ** documentation.  We had to figure it out via trial and error.
       */
-      for(i=5; i<strlen(zRefName) && zRefName[i]!='/'; i++){}
+      for(i=5; i<(int)strlen(zRefName) && zRefName[i]!='/'; i++){}
       gg.tagCommit = strncmp(&zRefName[5], "tags", 4)==0; /* pattern B */
       if( zRefName[i+1]!=0 ) zRefName += i+1;
       if( fossil_strcmp(zRefName, "master")==0 ) zRefName = ggit.zMasterName;
@@ -769,7 +773,7 @@ static void git_fast_import(FILE *pIn){
         if( pFile->isFrom==0 ) continue;
         pNew = import_add_file();
         pFile = &gg.aFile[i-1];
-        if( strlen(pFile->zName)>nFrom ){
+        if( (int)strlen(pFile->zName)>nFrom ){
           pNew->zName = mprintf("%s%s", zTo, pFile->zName+nFrom);
         }else{
           pNew->zName = fossil_strdup(zTo);
@@ -792,7 +796,7 @@ static void git_fast_import(FILE *pIn){
         if( pFile->isFrom==0 ) continue;
         pNew = import_add_file();
         pFile = &gg.aFile[i-1];
-        if( strlen(pFile->zName)>nFrom ){
+        if( (int)strlen(pFile->zName)>nFrom ){
           pNew->zName = mprintf("%s%s", zTo, pFile->zName+nFrom);
         }else{
           pNew->zName = fossil_strdup(zTo);
@@ -852,7 +856,7 @@ static struct{
   const char *zTags;          /* Name of tags folder in repo root */
   int lenTags;                /* String length of zTags */
   Bag newBranches;            /* Branches that were created in this revision */
-  int revFlag;                /* Add svn-rev-nn tags on every checkin */
+  int revFlag;                /* Add svn-rev-nn tags on every check-in */
   const char *zRevPre;        /* Prepended to revision tag names */
   const char *zRevSuf;        /* Appended to revision tag names */
   const char **azIgnTree;     /* NULL-terminated list of dirs to ignore */
@@ -1015,7 +1019,7 @@ static int svn_read_rec(FILE *pIn, SvnRecord *rec){
     rec->contentFlag = 1;
     nLen = atoi(zLen);
     blob_read_from_channel(&rec->content, pIn, nLen);
-    if( blob_size(&rec->content)!=nLen ){
+    if( (int)blob_size(&rec->content)!=nLen ){
       fossil_fatal("short read: got %d of %d bytes",
         blob_size(&rec->content), nLen
       );
@@ -1280,7 +1284,7 @@ static int svn_parse_path(char *zPath, char **zFile, int *type){
     unsigned nPath = strlen(zPath);
     for( pzIgnTree = gsvn.azIgnTree; *pzIgnTree; ++pzIgnTree ){
       const char *zIgn = *pzIgnTree;
-      int nIgn = strlen(zIgn);
+      unsigned nIgn = strlen(zIgn);
       if( strncmp(zPath, zIgn, nIgn) == 0
        && ( nPath == nIgn || (nPath > nIgn && zPath[nIgn] == '/')) ){
         return 0;
@@ -1691,15 +1695,15 @@ static void svn_dump_import(FILE *pIn){
 **                  --ignore-tree DIR  Ignores subtree rooted at DIR
 **
 ** Common Options:
-**   -i|--incremental     allow importing into an existing repository
-**   -f|--force           overwrite repository if already exists
-**   -q|--quiet           omit progress output
-**   --no-rebuild         skip the "rebuilding metadata" step
-**   --no-vacuum          skip the final VACUUM of the database file
-**   --rename-trunk NAME  use NAME as name of imported trunk branch
-**   --rename-branch PAT  rename all branch names using PAT pattern
-**   --rename-tag PAT     rename all tag names using PAT pattern
-**   -A|--admin-user NAME use NAME for the admin user 
+**   -i|--incremental     Allow importing into an existing repository
+**   -f|--force           Overwrite repository if already exists
+**   -q|--quiet           Omit progress output
+**   --no-rebuild         Skip the "rebuilding metadata" step
+**   --no-vacuum          Skip the final VACUUM of the database file
+**   --rename-trunk NAME  Use NAME as name of imported trunk branch
+**   --rename-branch PAT  Rename all branch names using PAT pattern
+**   --rename-tag PAT     Rename all tag names using PAT pattern
+**   -A|--admin-user NAME Use NAME for the admin user
 **
 ** The --incremental option allows an existing repository to be extended
 ** with new content.  The --rename-* options may be useful to avoid name
@@ -1717,7 +1721,11 @@ static void svn_dump_import(FILE *pIn){
 ** The --attribute option takes a quoted string argument comprised of a
 ** Git committer email and the username to be attributed to corresponding
 ** check-ins in the Fossil repository. This option can be repeated. For
-** example, --attribute "drh@sqlite.org drh" --attribute "xyz@abc.net X"
+** example, --attribute "drh@sqlite.org drh" --attribute "xyz@abc.net X".
+** Attributions are persisted to the repository so that subsequent
+** 'fossil git export' operations attribute Fossil commits to corresponding
+** 'Git Committer <git@committer.com>' users, and incremental imports with
+** 'fossil import --git --incremental' use previous --attribute records.
 **
 ** See also: export
 */
@@ -1953,9 +1961,11 @@ void import_cmd(void){
     if(ggit.nGitAttr > 0) {
       int idx;
       db_unprotect(PROTECT_ALL);
-      db_multi_exec(
-        "CREATE TABLE fx_git(user TEXT, email TEXT UNIQUE);"
-      );
+      if( !db_table_exists("repository", "fx_git") ){
+        db_multi_exec(
+          "CREATE TABLE fx_git(user TEXT, email TEXT UNIQUE);"
+        );
+      }
       for(idx = 0; idx < ggit.nGitAttr; ++idx ){
         db_multi_exec(
             "INSERT OR IGNORE INTO fx_git(user, email) VALUES(%Q, %Q)",
@@ -2024,5 +2034,6 @@ void import_cmd(void){
     fossil_print("server-id:  %s\n", db_get("server-code", 0));
     zPassword = db_text(0, "SELECT pw FROM user WHERE login=%Q", g.zLogin);
     fossil_print("admin-user: %s (password is \"%s\")\n", g.zLogin, zPassword);
+    hash_user_password(g.zLogin);
   }
 }
