@@ -51,6 +51,21 @@
 #endif
 #include <time.h>
 
+/*
+** Compute an appropriate Anti-CSRF token into g.zCsrfToken[].
+*/
+static void login_create_csrf_secret(const char *zSeed){
+  unsigned char zResult[20];
+  int i;
+
+  sha1sum_binary(zSeed, zResult);
+  for(i=0; i<sizeof(g.zCsrfToken)-1; i++){
+    g.zCsrfToken[i] = "abcdefghijklmnopqrstuvwxyz"
+                      "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                      "0123456789-/"[zResult[i]%64];
+  }
+  g.zCsrfToken[i] = 0;
+}
 
 /*
 ** Return the login-group name.  Or return 0 if this repository is
@@ -146,7 +161,7 @@ int login_is_valid_anonymous(
   zPw = captcha_decode((unsigned int)atoi(zCS));
   if( fossil_stricmp(zPw, zPassword)!=0 ) return 0;
   uid = db_int(0, "SELECT uid FROM user WHERE login='anonymous'"
-                  " AND length(pw)>0 AND length(cap)>0");
+                  " AND octet_length(pw)>0 AND octet_length(cap)>0");
   return uid;
 }
 
@@ -212,7 +227,7 @@ int login_search_uid(const char **pzUsername, const char *zPasswd){
   int uid = db_int(0,
     "SELECT uid FROM user"
     " WHERE login=%Q"
-    "   AND length(cap)>0 AND length(pw)>0"
+    "   AND octet_length(cap)>0 AND octet_length(pw)>0"
     "   AND login NOT IN ('anonymous','nobody','developer','reader')"
     "   AND (pw=%Q OR (length(pw)<>40 AND pw=%Q))"
     "   AND (info NOT LIKE '%%expires 20%%'"
@@ -585,7 +600,7 @@ void login_page(void){
   zPasswd = P("p");
   anonFlag = g.zLogin==0 && PB("anon");
   /* Handle log-out requests */
-  if( P("out") ){
+  if( P("out") && cgi_csrf_safe(2) ){
     login_clear_login_data();
     redirect_to_g();
     return;
@@ -600,6 +615,7 @@ void login_page(void){
   /* Deal with password-change requests */
   if( g.perm.Password && zPasswd
    && (zNew1 = P("n1"))!=0 && (zNew2 = P("n2"))!=0
+   && cgi_csrf_safe(2)
   ){
     /* If there is not a "real" login, we cannot change any password. */
     if( g.zLogin ){
@@ -739,10 +755,10 @@ void login_page(void){
     form_begin(0, "%R/login");
   }
   if( zGoto ){
-    @ <input type="hidden" name="g" value="%h(zGoto)" />
+    @ <input type="hidden" name="g" value="%h(zGoto)">
   }
   if( anonFlag ){
-    @ <input type="hidden" name="anon" value="1" />
+    @ <input type="hidden" name="anon" value="1">
   }
   if( g.zLogin ){
     @ <p>Currently logged in as <b>%h(g.zLogin)</b>.
@@ -777,7 +793,7 @@ void login_page(void){
     @ <tr>
     @  <td class="form_label" id="pswdlabel">Password:</td>
     @  <td><input aria-labelledby="pswdlabel" type="password" id="p" \
-    @ name="p" value="" size="30" />\
+    @ name="p" value="" size="30">\
     if( zAnonPw && !noAnon ){
       captcha_speakit_button(uSeed, "Speak password for \"anonymous\"");
     }
@@ -811,7 +827,7 @@ void login_page(void){
       int bAutoCaptcha = db_get_boolean("auto-captcha", 0);
       char *zCaptcha = captcha_render(zDecoded);
   
-      @ <p><input type="hidden" name="cs" value="%u(uSeed)" />
+      @ <p><input type="hidden" name="cs" value="%u(uSeed)">
       @ Visitors may enter <b>anonymous</b> as the user-ID with
       @ the 8-character hexadecimal password shown below:</p>
       @ <div class="captcha"><table class="captcha"><tr><td>\
@@ -820,7 +836,7 @@ void login_page(void){
       @ </pre></td></tr></table>
       if( bAutoCaptcha ) {
          @ <input type="button" value="Fill out captcha" id='autofillButton' \
-         @ data-af='%s(zDecoded)' />
+         @ data-af='%s(zDecoded)'>
          builtin_request_js("login.js");
       }
       @ </div>
@@ -850,12 +866,12 @@ void login_page(void){
       @ size="30"/></td></tr>
       @ <tr><td class="form_label" id="newpw">New Password:</td>
       @ <td><input aria-labelledby="newpw" type="password" name="n1" \
-      @ size="30" /> Suggestion: %z(zRPW)</td></tr>
+      @ size="30"> Suggestion: %z(zRPW)</td></tr>
       @ <tr><td class="form_label" id="reppw">Repeat New Password:</td>
       @ <td><input aria-labledby="reppw" type="password" name="n2" \
-      @ size="30" /></td></tr>
+      @ size="30"></td></tr>
       @ <tr><td></td>
-      @ <td><input type="submit" value="Change Password" /></td></tr>
+      @ <td><input type="submit" value="Change Password"></td></tr>
       @ </table>
       @ </form>
     }
@@ -1087,12 +1103,12 @@ void login_resetpw(void){
   @ <table>
   @ <tr><td class="form_label" id="newpw">New Password:</td>
   @ <td><input aria-labelledby="newpw" type="password" name="n1" \
-  @ size="30" /> Suggestion: %z(zRPW)</td></tr>
+  @ size="30"> Suggestion: %z(zRPW)</td></tr>
   @ <tr><td class="form_label" id="reppw">Repeat New Password:</td>
   @ <td><input aria-labledby="reppw" type="password" name="n2" \
-  @ size="30" /></td></tr>
+  @ size="30"></td></tr>
   @ <tr><td></td>
-  @ <td><input type="submit" value="Change Password" /></td></tr>
+  @ <td><input type="submit" value="Change Password"></td></tr>
   @ </table>
   @ </form>
   style_finish_page();
@@ -1136,8 +1152,8 @@ static int login_transfer_credentials(
     zSQL = mprintf(
       "SELECT cexpire FROM user"
       " WHERE login=%Q"
-      "   AND length(cap)>0"
-      "   AND length(pw)>0"
+      "   AND octet_length(cap)>0"
+      "   AND octet_length(pw)>0"
       "   AND cexpire>julianday('now')"
       "   AND constant_time_cmp(cookie,%Q)=0",
       zLogin, zHash
@@ -1190,8 +1206,8 @@ static int login_find_user(
     "SELECT uid FROM user"
     " WHERE login=%Q"
     "   AND cexpire>julianday('now')"
-    "   AND length(cap)>0"
-    "   AND length(pw)>0"
+    "   AND octet_length(cap)>0"
+    "   AND octet_length(pw)>0"
     "   AND constant_time_cmp(cookie,%Q)=0",
     zLogin, zCookie
   );
@@ -1290,6 +1306,7 @@ void login_check_credentials(void){
    && db_get_int("localauth",0)==0
    && P("HTTPS")==0
   ){
+    char *zSeed;
     if( g.localOpen ) zLogin = db_lget("default-user",0);
     if( zLogin!=0 ){
       uid = db_int(0, "SELECT uid FROM user WHERE login=%Q", zLogin);
@@ -1300,7 +1317,10 @@ void login_check_credentials(void){
     zCap = "sxy";
     g.noPswd = 1;
     g.isHuman = 1;
-    sqlite3_snprintf(sizeof(g.zCsrfToken), g.zCsrfToken, "localhost");
+    zSeed = db_text("??", "SELECT uid||quote(login)||quote(pw)||quote(cookie)"
+                          "  FROM user WHERE uid=%d", uid);
+    login_create_csrf_secret(zSeed);
+    fossil_free(zSeed);
   }
 
   /* Check the login cookie to see if it matches a known valid user.
@@ -1337,8 +1357,8 @@ void login_check_credentials(void){
       if( fossil_strcmp(zHash, blob_str(&b))==0 ){
         uid = db_int(0,
             "SELECT uid FROM user WHERE login='anonymous'"
-            " AND length(cap)>0"
-            " AND length(pw)>0"
+            " AND octet_length(cap)>0"
+            " AND octet_length(pw)>0"
             " AND %.17g+0.25>julianday('now')",
             rTime
         );
@@ -1355,7 +1375,7 @@ void login_check_credentials(void){
         if( uid ) record_login_attempt(zUser, zIpAddr, 1);
       }
     }
-    sqlite3_snprintf(sizeof(g.zCsrfToken), g.zCsrfToken, "%.10s", zHash);
+    login_create_csrf_secret(zHash);
   }
 
   /* If no user found and the REMOTE_USER environment variable is set,
@@ -1365,7 +1385,8 @@ void login_check_credentials(void){
     const char *zRemoteUser = P("REMOTE_USER");
     if( zRemoteUser && db_get_boolean("remote_user_ok",0) ){
       uid = db_int(0, "SELECT uid FROM user WHERE login=%Q"
-                      " AND length(cap)>0 AND length(pw)>0", zRemoteUser);
+                      " AND octet_length(cap)>0 AND octet_length(pw)>0",
+                      zRemoteUser);
     }
   }
 
@@ -1404,7 +1425,7 @@ void login_check_credentials(void){
       uid = -1;
       zCap = "";
     }
-    sqlite3_snprintf(sizeof(g.zCsrfToken), g.zCsrfToken, "none");
+    login_create_csrf_secret("none");
   }
 
   login_set_uid(uid, zCap);
@@ -1792,7 +1813,7 @@ void login_needed(int anonOk){
 void login_anonymous_available(void){
   if( !g.perm.Hyperlink && g.anon.Hyperlink ){
     const char *zUrl = PD("PATH_INFO", "");
-    @ <p>Many <span class="disabled">hyperlinks are disabled.</span><br />
+    @ <p>Many <span class="disabled">hyperlinks are disabled.</span><br>
     @ Use <a href="%R/login?anon=1&amp;g=%T(zUrl)">anonymous login</a>
     @ to enable hyperlinks.</p>
   }
@@ -1803,23 +1824,7 @@ void login_anonymous_available(void){
 ** as a hidden element of the form.
 */
 void login_insert_csrf_secret(void){
-  @ <input type="hidden" name="csrf" value="%s(g.zCsrfToken)" />
-}
-
-/*
-** Before using the results of a form, first call this routine to verify
-** that this Anti-CSRF token is present and is valid.  If the Anti-CSRF token
-** is missing or is incorrect, that indicates a cross-site scripting attack.
-** If the event of an attack is detected, an error message is generated and
-** all further processing is aborted.
-*/
-void login_verify_csrf_secret(void){
-  if( g.okCsrf ) return;
-  if( fossil_strcmp(P("csrf"), g.zCsrfToken)==0 ){
-    g.okCsrf = 1;
-    return;
-  }
-  fossil_fatal("Cross-site request forgery attempt");
+  @ <input type="hidden" name="csrf" value="%s(g.zCsrfToken)">
 }
 
 /*
@@ -1964,6 +1969,7 @@ void register_page(void){
     return;
   }
   zPerms = db_get("default-perms", "u");
+  login_check_credentials();
 
   /* Prompt the user for email alerts if this repository is configured for
   ** email alerts and if the default permissions include "7" */
@@ -1979,7 +1985,7 @@ void register_page(void){
   zDName = PDT("dn","");
 
   /* Verify user imputs */
-  if( P("new")==0 || !cgi_csrf_safe(1) ){
+  if( P("new")==0 || !cgi_csrf_safe(2) ){
     /* This is not a valid form submission.  Fall through into
     ** the form display */
   }else if( (captchaIsCorrect = captcha_is_correct(1))==0 ){
@@ -1991,6 +1997,13 @@ void register_page(void){
   }else if( sqlite3_strglob("*[^-a-zA-Z0-9_.]*",zUserID)==0 ){
     iErrLine = 1;
     zErr = "User ID may not contain spaces or special characters.";
+  }else if( sqlite3_strlike("anonymous%", zUserID, 0)==0 
+         || sqlite3_strlike("nobody%", zUserID, 0)==0
+         || sqlite3_strlike("reader%", zUserID, 0)==0
+         || sqlite3_strlike("developer%", zUserID, 0)==0
+  ){
+    iErrLine = 1;
+    zErr = "This User ID is reserved. Choose something different.";
   }else if( zDName[0]==0 ){
     iErrLine = 2;
     zErr = "Required";
@@ -2125,9 +2138,9 @@ void register_page(void){
   g.perm.Hyperlink = 1;  /* Artificially enable hyperlinks */
   form_begin(0, "%R/register");
   if( P("g") ){
-    @ <input type="hidden" name="g" value="%h(P("g"))" />
+    @ <input type="hidden" name="g" value="%h(P("g"))">
   }
-  @ <p><input type="hidden" name="captchaseed" value="%u(uSeed)" />
+  @ <p><input type="hidden" name="captchaseed" value="%u(uSeed)">
   @ <table class="login_out">
   @ <tr>
   @   <td class="form_label" align="right" id="uid">User ID:</td>
@@ -2154,7 +2167,7 @@ void register_page(void){
   if( iErrLine==3 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span>
     if( uid>0 && login_self_password_reset_available() ){
-      @ <br />
+      @ <br>
       @ <input type="submit" name="pwreset" \
       @ value="Request Password Reset For %h(zEAddr)">
     }
@@ -2202,7 +2215,7 @@ void register_page(void){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
   }
   @ <tr><td></td>
-  @ <td><input type="submit" name="new" value="Register" /></td></tr>
+  @ <td><input type="submit" name="new" value="Register"></td></tr>
   @ </table>
   @ <div class="captcha"><table class="captcha"><tr><td><pre class="captcha">
   @ %h(zCaptcha)
@@ -2253,6 +2266,10 @@ void login_reqpwreset_page(void){
   if( !cgi_csrf_safe(1) || P("reqpwreset")==0 ){
     /* This is the initial display of the form.  No processing or error
     ** checking is to be done. Fall through into the form display
+    **
+    ** cgi_csrf_safe():  Nothing interesting happens on this page without
+    ** a valid captcha solution, so we only need to check referrer and that
+    ** the request is a POST.
     */
   }else if( (captchaIsCorrect = captcha_is_correct(1))==0 ){
     iErrLine = 2;
@@ -2330,8 +2347,8 @@ void login_reqpwreset_page(void){
   /* Print out the registration form. */
   g.perm.Hyperlink = 1;  /* Artificially enable hyperlinks */
   form_begin(0, "%R/reqpwreset");
-  @ <p><input type="hidden" name="captchaseed" value="%u(uSeed)" />
-  @ <p><input type="hidden" name="reqpwreset" value="1" />
+  @ <p><input type="hidden" name="captchaseed" value="%u(uSeed)">
+  @ <p><input type="hidden" name="reqpwreset" value="1">
   @ <table class="login_out">
   @ <tr>
   @   <td class="form_label" align="right" id="emaddr">Email Address:</td>
