@@ -232,16 +232,18 @@ int pikchr_process(const char * zIn, int pikFlags, int thFlags,
 }
 
 /*
-** WEBPAGE: pikchrshow
+** Legacy impl of /pikchrshow. pikchrshow_page() will delegate to
+** this one if the "legacy" or "ajax" request arguments are set.
 **
 ** A pikchr code editor and previewer, allowing users to experiment
 ** with pikchr code or prototype it for use in copy/pasting into forum
-** posts, wiki pages, or embedded docs.
-**
-** It optionally accepts a p=pikchr-script-code URL parameter or POST
-** value to pre-populate the editor with that code.
+** posts, wiki pages, or embedded docs. This version of pikchrshow
+** uses JavaScript to send pikchr code to the server for
+** processing. The newer /pikchrshow applications runs pikchr on the
+** client machine, without the need for back-and-forth network
+** traffic.
 */
-void pikchrshow_page(void){
+void pikchrshowcs_page(void){
   const char *zContent = 0;
   int isDark;              /* true if the current skin is "dark" */
   int pikFlags =
@@ -251,7 +253,11 @@ void pikchrshow_page(void){
 
   login_check_credentials();
   if( !g.perm.RdWiki && !g.perm.Read && !g.perm.RdForum ){
-    cgi_redirectf("%R/login?g=pikchrshow");
+    cgi_redirectf("%R/login?g=pikchrshowcs");
+  }
+  if(P("wasm")){
+    pikchrshow_page();
+    return;
   }
   zContent = PD("content",P("p"));
   if(P("ajax")!=0){
@@ -281,7 +287,7 @@ void pikchrshow_page(void){
       "arrow <-> down from last box.s\n"
       "box same \"Pikchr\" \"Formatter\" \"(pikchr.c)\" fit\n";
   }
-  style_header("PikchrShow");
+  style_header("PikchrShow Client/Server");
   CX("<style>"); {
     CX("div.content { padding-top: 0.5em }\n");
     CX("#sbs-wrapper {"
@@ -338,7 +344,7 @@ void pikchrshow_page(void){
     CX(".dragover {border: 3px dotted rgba(0,255,0,0.6)}\n");
   } CX("</style>");
   CX("<div>Input pikchr code and tap Preview (or Shift-Enter) to render "
-     "it:</div>");
+     "it. <a href='?wasm'>Switch to WASM mode</a>.</div>");
   CX("<div id='sbs-wrapper'>"); {
     CX("<div id='pikchrshow-form'>"); {
       CX("<textarea id='content' name='content' rows='15'>"
@@ -379,6 +385,137 @@ void pikchrshow_page(void){
   style_finish_page();
 }
 
+/*
+** WEBPAGE: pikchrshow
+**
+** A pikchr code editor and previewer, allowing users to experiment
+** with pikchr code or prototype it for use in copy/pasting into forum
+** posts, wiki pages, or embedded docs. This version of pikchrshow
+** uses WebAssembly to run entirely in the client browser, without a
+** need for back-and-forth client/server traffic to perform the
+** rendering. The "legacy" version of this application, which sends
+** all input to the server for rendering, can be accessed by adding
+** the "legacy" URL argument.
+**
+** It optionally accepts a p=pikchr-script-code URL parameter or POST
+** value to pre-populate the editor with that code.
+*/
+void pikchrshow_page(void){
+  const char *zContent = 0;
+
+  if(P("legacy") || P("ajax")){
+    pikchrshowcs_page();
+    return;
+  }
+  login_check_credentials();
+  if( !g.perm.RdWiki && !g.perm.Read && !g.perm.RdForum ){
+    cgi_redirectf("%R/login?g=pikchrshow");
+  }
+  style_emit_noscript_for_js_page();
+  style_header("PikchrShow");
+  zContent = PD("content",P("p"));
+  if(!zContent){
+    zContent = "arrow right 200% \"Markdown\" \"Source\"\n"
+      "box rad 10px \"Markdown\" \"Formatter\" \"(markdown.c)\" fit\n"
+      "arrow right 200% \"HTML+SVG\" \"Output\"\n"
+      "arrow <-> down from last box.s\n"
+      "box same \"Pikchr\" \"Formatter\" \"(pikchr.c)\" fit\n";
+  }
+  /* Wasm load/init progress widget... */
+  CX("<div class='emscripten'>"); {
+    CX("<figure id='module-spinner'>");
+      CX("<div class='spinner'></div>");
+      CX("<div class='center'><strong>Initializing app...</strong></div>");
+      CX("<div class='center'>");
+        CX("On a slow internet connection this may take a moment.  If this ");
+        CX("message displays for \"a long time\", initialization may have ");
+        CX("failed and the JavaScript console may contain clues as to why. ");
+      CX("</div>");
+      CX("<div><a href='?legacy'>Switch to legacy mode</a></div>");
+    CX("</figure>");
+    CX("<div class='emscripten' id='module-status'>Downloading...</div>");
+    CX("<progress value='0' max='100' id='module-progress' hidden='1'>"
+       "</progress>");
+  } CX("</div><!-- .emscripten -->");
+  /* Main view... */
+  CX("<div id='view-split' class='app-view initially-hidden'>"); {
+    CX("<fieldset class='options collapsible'>"); {
+      CX("<legend><button class='fieldset-toggle'>Options</button></legend>");
+      CX("<div>");
+      CX("<span class='labeled-input'>");
+        CX("<input type='checkbox' id='opt-cb-sbs' ");
+        CX("data-csstgt='#main-wrapper' ");
+        CX("data-cssclass='side-by-side' ");
+        CX("data-config='sideBySide'>");
+        CX("<label for='opt-cb-sbs'>Side-by-side</label>");
+      CX("</span>");
+      CX("<span class='labeled-input'>");
+        CX("<input type='checkbox' id='opt-cb-swapio' ");
+        CX("data-csstgt='#main-wrapper' ");
+        CX("data-cssclass='swapio' ");
+        CX("data-config='swapInOut'>");
+        CX("<label for='opt-cb-swapio'>Swap in/out</label>");
+      CX("</span>");
+      CX("<span class='labeled-input'>");
+        CX("<input type='checkbox' id='opt-cb-autofit' ");
+        CX("data-config='renderAutofit'>");
+        CX("<label for='opt-cb-autofit' "
+           "title='Attempt to scale SVG to fit viewport. "
+           "Whether it will work depends in part on the size "
+           "and shape of the image and the viewport.'"
+           ">Auto-fit SVG</label>");
+      CX("</span>");
+      CX("<span class='labeled-input'>");
+        CX("<input type='checkbox' id='opt-cb-autorender' ");
+        CX("data-csstgt='#main-wrapper' ");
+        CX("data-cssclass='auto-render' ");
+        CX("data-config='renderWhileTyping'>");
+        CX("<label for='opt-cb-autorender'>Render while typing</label>");
+      CX("</span>");
+      CX("<span class='labeled-input'>");
+        CX("<a href='?legacy'>Legacy mode</a>");
+      CX("</span>");
+      CX("</div><!-- options wrapper -->");
+    } CX("</fieldset>");
+    CX("<div id='main-wrapper' class=''>"); {
+      CX("<fieldset class='zone-wrapper input'>"); {
+        CX("<legend><div class='button-bar'>");
+          CX("<button id='btn-render' "
+             "title='Ctrl-Enter/Shift-Enter'>Render</button>");
+          CX("<button id='btn-clear'>Clear Input</button>");
+        CX("</div></legend>");
+        CX("<div><textarea id='input'");
+          CX("placeholder='Pikchr input. Ctrl-enter/shift-enter runs it.'>");
+          CX("/**\n");
+          CX("  Use ctrl-enter or shift-enter to execute\n");
+          CX("  pikchr code. If only a subset is currently\n");
+          CX("  selected, only that part is evaluated.\n*/\n");
+        CX("%s</textarea></div>",zContent/*safe-for-%s*/);
+      } CX("</fieldset><!-- .zone-wrapper.input -->");
+      CX("<fieldset class='zone-wrapper output'>"); {
+        CX("<legend><div class='button-bar'>");
+          CX("<button id='btn-render-mode'>Render Mode</button> ");
+          CX("<span style='white-space:nowrap'>"
+             "<span id='preview-copy-button' "
+             "title='Tap to copy to clipboard.'></span>"
+             "<label for='preview-copy-button' "
+             "title='Tap to copy to clipboard.'></label>"
+             "</span>");
+        CX("</div></legend>");
+        CX("<div id='pikchr-output-wrapper'>");
+          CX("<div id='pikchr-output'></div>");
+          CX("<textarea class='hidden' id='pikchr-output-text'></textarea>");
+        CX("</div>");
+      } CX("</fieldset> <!-- .zone-wrapper.output -->");
+    } CX("</div><!-- #main-wrapper -->");
+  } CX("</div><!-- #view-split -->");
+  builtin_fossil_js_bundle_or("dom", "storage", "copybutton", NULL);
+  builtin_request_js("fossil.page.pikchrshowasm.js");
+  builtin_fulfill_js_requests();
+  style_finish_page();
+}
+
+
 /*
 ** COMMAND: pikchr*
 **
@@ -390,21 +527,20 @@ void pikchrshow_page(void){
 ** streams.
 **
 ** Options:
-**
 **    -div       On success, add a DIV wrapper around the
 **               resulting SVG output which limits its max-width to
-**               its computed maximum ideal size.
+**               its computed maximum ideal size
 **
-**    -div-indent  Like -div but indent the div.
+**    -div-indent  Like -div but indent the div
 **
-**    -div-center  Like -div but center the div.
+**    -div-center  Like -div but center the div
 **
-**    -div-left    Like -div but float the div left.
+**    -div-left    Like -div but float the div left
 **
-**    -div-right   Like -div but float the div right.
+**    -div-right   Like -div but float the div right
 **
 **    -div-toggle  Set the 'toggle' CSS class on the div (used by the
-**                 JavaScript-side post-processor).
+**                 JavaScript-side post-processor)
 **
 **    -div-source  Set the 'source' CSS class on the div, which tells
 **                 CSS to hide the SVG and reveal the source by default.
@@ -414,7 +550,7 @@ void pikchrshow_page(void){
 **               by -div-source.
 **                
 **
-**    -th        Process the input using TH1 before passing it to pikchr.
+**    -th        Process the input using TH1 before passing it to pikchr
 **
 **    -th-novar  Disable $var and $<var> TH1 processing. Use this if the
 **               pikchr script uses '$' for its own purposes and that
@@ -422,9 +558,9 @@ void pikchrshow_page(void){
 **               of TH1 script blocks. Code in such blocks is unaffected.
 **
 **    -th-nosvg  When using -th, output the post-TH1'd script
-**               instead of the pikchr-rendered output.
+**               instead of the pikchr-rendered output
 **
-**    -th-trace  Trace TH1 execution (for debugging purposes).
+**    -th-trace  Trace TH1 execution (for debugging purposes)
 **
 **
 ** The -div-indent/center/left/right flags may not be combined.
@@ -432,7 +568,7 @@ void pikchrshow_page(void){
 ** TH1-related Notes and Caveats:
 **
 ** If the -th flag is used, this command must open a fossil database
-** for certain functionality to work (via a checkout or the -R REPO
+** for certain functionality to work (via a check-out or the -R REPO
 ** flag). If opening a db fails, execution will continue but any TH1
 ** commands which require a db will trigger a fatal error.
 **
