@@ -807,6 +807,7 @@ void taglist_page(void){
   if( !g.perm.Read ){
     login_needed(g.anon.Read);
   }
+  cgi_check_for_malice();
   login_anonymous_available();
   style_header("Tags");
   style_adunit_config(ADUNIT_RIGHT_OK);
@@ -888,7 +889,7 @@ void tagtimeline_page(void){
   if( PB("ubg")!=0 ) tmFlags |= TIMELINE_UCOLOR;
   www_print_timeline(&q, tmFlags, 0, 0, 0, 0, 0, 0);
   db_finalize(&q);
-  @ <br />
+  @ <br>
   style_finish_page();
 }
 
@@ -905,3 +906,34 @@ int rid_has_tag(int rid, int tagId){
      rid, tagId
   );
 }
+
+
+/*
+** Returns tagxref.rowid if the given blob.rid has a tagxref.rid entry
+** of an active (non-cancelled) tag matching the given rid and tag
+** name string, else returns 0. Note that this function does not
+** distinguish between a non-existent tag and a cancelled tag.
+**
+** Design note: the return value is the tagxref.rowid because that
+** gives us an easy way to fetch the value of the tag later on, if
+** needed.
+*/
+int rid_has_active_tag_name(int rid, const char *zTagName){
+  static Stmt q = empty_Stmt_m;
+  int rc;
+
+  assert( 0 != zTagName );
+  if( !q.pStmt ){
+    db_static_prepare(&q,
+       "SELECT x.rowid FROM tagxref x, tag t"
+       " WHERE x.rid=$rid AND x.tagtype>0 "
+       " AND x.tagid=t.tagid"
+       " AND t.tagname=$tagname"
+    );
+  }
+  db_bind_int(&q, "$rid", rid);
+  db_bind_text(&q, "$tagname", zTagName);
+  rc = (SQLITE_ROW==db_step(&q)) ? db_column_int(&q, 0) : 0;
+  db_reset(&q);
+  return rc;
+}
