@@ -197,6 +197,7 @@ enum markup_t {
   MARKUP_COLGROUP,
   MARKUP_DD,
   MARKUP_DEL,
+  MARKUP_DETAILS,
   MARKUP_DFN,
   MARKUP_DIV,
   MARKUP_DL,
@@ -231,6 +232,7 @@ enum markup_t {
   MARKUP_STRIKE,
   MARKUP_STRONG,
   MARKUP_SUB,
+  MARKUP_SUMMARY,
   MARKUP_SUP,
   MARKUP_TABLE,
   MARKUP_TBODY,
@@ -288,10 +290,9 @@ static const struct AllowedMarkup {
                     AMSK_ID|AMSK_CLASS|AMSK_STYLE|AMSK_TITLE },
  { "address",       MARKUP_ADDRESS,      MUTYPE_BLOCK,         AMSK_STYLE },
  { "article",       MARKUP_HTML5_ARTICLE, MUTYPE_BLOCK,
-                                            AMSK_ID|AMSK_CLASS|AMSK_STYLE },
+                    AMSK_ID|AMSK_CLASS|AMSK_STYLE },
  { "aside",         MARKUP_HTML5_ASIDE,  MUTYPE_BLOCK,
-                                            AMSK_ID|AMSK_CLASS|AMSK_STYLE },
-
+                    AMSK_ID|AMSK_CLASS|AMSK_STYLE },
  { "b",             MARKUP_B,            MUTYPE_FONT,          AMSK_STYLE },
  { "big",           MARKUP_BIG,          MUTYPE_FONT,          AMSK_STYLE },
  { "blockquote",    MARKUP_BLOCKQUOTE,   MUTYPE_BLOCK,         AMSK_STYLE },
@@ -305,6 +306,8 @@ static const struct AllowedMarkup {
                     AMSK_ALIGN|AMSK_CLASS|AMSK_COLSPAN|AMSK_WIDTH|AMSK_STYLE},
  { "dd",            MARKUP_DD,           MUTYPE_LI,            AMSK_STYLE },
  { "del",           MARKUP_DEL,          MUTYPE_FONT,          AMSK_STYLE },
+ { "details",       MARKUP_DETAILS,      MUTYPE_BLOCK,
+                    AMSK_ID|AMSK_CLASS|AMSK_STYLE },
  { "dfn",           MARKUP_DFN,          MUTYPE_FONT,          AMSK_STYLE },
  { "div",           MARKUP_DIV,          MUTYPE_BLOCK,
                     AMSK_ID|AMSK_CLASS|AMSK_STYLE },
@@ -315,8 +318,7 @@ static const struct AllowedMarkup {
  { "font",          MARKUP_FONT,         MUTYPE_FONT,
                     AMSK_COLOR|AMSK_FACE|AMSK_SIZE|AMSK_STYLE },
  { "footer",        MARKUP_HTML5_FOOTER, MUTYPE_BLOCK,
-                                            AMSK_ID|AMSK_CLASS|AMSK_STYLE },
-
+                    AMSK_ID|AMSK_CLASS|AMSK_STYLE },
  { "h1",            MARKUP_H1,           MUTYPE_BLOCK,
                     AMSK_ALIGN|AMSK_CLASS|AMSK_STYLE  },
  { "h2",            MARKUP_H2,           MUTYPE_BLOCK,
@@ -329,10 +331,8 @@ static const struct AllowedMarkup {
                     AMSK_ALIGN|AMSK_CLASS|AMSK_STYLE  },
  { "h6",            MARKUP_H6,           MUTYPE_BLOCK,
                     AMSK_ALIGN|AMSK_CLASS|AMSK_STYLE  },
-
  { "header",        MARKUP_HTML5_HEADER, MUTYPE_BLOCK,
-                                            AMSK_ID|AMSK_CLASS|AMSK_STYLE },
-
+                    AMSK_ID|AMSK_CLASS|AMSK_STYLE },
  { "hr",            MARKUP_HR,           MUTYPE_SINGLE,
                     AMSK_ALIGN|AMSK_COLOR|AMSK_SIZE|AMSK_WIDTH|
                     AMSK_STYLE|AMSK_CLASS  },
@@ -345,7 +345,7 @@ static const struct AllowedMarkup {
  { "li",            MARKUP_LI,           MUTYPE_LI,
                     AMSK_TYPE|AMSK_VALUE|AMSK_STYLE  },
  { "nav",           MARKUP_HTML5_NAV,    MUTYPE_BLOCK,
-                                            AMSK_ID|AMSK_CLASS|AMSK_STYLE },
+                    AMSK_ID|AMSK_CLASS|AMSK_STYLE },
  { "nobr",          MARKUP_NOBR,         MUTYPE_FONT,          0  },
  { "nowiki",        MARKUP_NOWIKI,       MUTYPE_SPECIAL,       0  },
  { "ol",            MARKUP_OL,           MUTYPE_LIST,
@@ -356,13 +356,15 @@ static const struct AllowedMarkup {
  { "s",             MARKUP_S,            MUTYPE_FONT,          AMSK_STYLE },
  { "samp",          MARKUP_SAMP,         MUTYPE_FONT,          AMSK_STYLE },
  { "section",       MARKUP_HTML5_SECTION, MUTYPE_BLOCK,
-                                            AMSK_ID|AMSK_CLASS|AMSK_STYLE },
+                    AMSK_ID|AMSK_CLASS|AMSK_STYLE },
  { "small",         MARKUP_SMALL,        MUTYPE_FONT,          AMSK_STYLE },
  { "span",          MARKUP_SPAN,         MUTYPE_BLOCK,
                     AMSK_ALIGN|AMSK_CLASS|AMSK_STYLE  },
  { "strike",        MARKUP_STRIKE,       MUTYPE_FONT,          AMSK_STYLE },
  { "strong",        MARKUP_STRONG,       MUTYPE_FONT,          AMSK_STYLE },
  { "sub",           MARKUP_SUB,          MUTYPE_FONT,          AMSK_STYLE },
+ { "summary",       MARKUP_SUMMARY,      MUTYPE_BLOCK,
+                    AMSK_ALIGN|AMSK_CLASS|AMSK_STYLE  },
  { "sup",           MARKUP_SUP,          MUTYPE_FONT,          AMSK_STYLE },
  { "table",         MARKUP_TABLE,        MUTYPE_TABLE,
                     AMSK_ALIGN|AMSK_BGCOLOR|AMSK_BORDER|AMSK_CELLPADDING|
