@@ -558,7 +558,7 @@ writeln "\t\$(XTCC) -c \$(SRCDIR)/th_lang.c -o \$@\n"
 writeln "\$(OBJDIR)/th_tcl.o:\t\$(SRCDIR)/th_tcl.c"
 writeln "\t\$(XTCC) -c \$(SRCDIR)/th_tcl.c -o \$@\n"
 
-writeln {
+writeln [string map [list <<<NEXT_LINE>>> \\] {
 $(OBJDIR)/pikchr.o:	$(SRCDIR_extsrc)/pikchr.c
 	$(XTCC) $(PIKCHR_OPTIONS) -c $(SRCDIR_extsrc)/pikchr.c -o $@
 
@@ -566,23 +566,66 @@ $(OBJDIR)/cson_amalgamation.o: $(SRCDIR_extsrc)/cson_amalgamation.c
 	$(XTCC) -c $(SRCDIR_extsrc)/cson_amalgamation.c -o $@
 
 $(SRCDIR_extsrc)/pikchr.js: $(SRCDIR_extsrc)/pikchr.c
-	$(EMCC_WRAPPER) -o $@ $(EMCC_OPT) --no-entry \
-        -sEXPORTED_RUNTIME_METHODS=cwrap,setValue,getValue,stackSave,stackRestore \
-        -sEXPORTED_FUNCTIONS=_pikchr $(SRCDIR_extsrc)/pikchr.c \
-        -sENVIRONMENT=web \
-        -sMODULARIZE \
-        -sEXPORT_NAME=initPikchrModule \
+	$(EMCC_WRAPPER) -o $@ $(EMCC_OPT) --no-entry <<<NEXT_LINE>>>
+        -sEXPORTED_RUNTIME_METHODS=cwrap,setValue,getValue,stackSave,stackRestore <<<NEXT_LINE>>>
+        -sEXPORTED_FUNCTIONS=_pikchr $(SRCDIR_extsrc)/pikchr.c <<<NEXT_LINE>>>
+        -sENVIRONMENT=web <<<NEXT_LINE>>>
+        -sMODULARIZE <<<NEXT_LINE>>>
+        -sEXPORT_NAME=initPikchrModule <<<NEXT_LINE>>>
         --minify 0
 	@chmod -x $(SRCDIR_extsrc)/pikchr.wasm
 wasm: $(SRCDIR_extsrc)/pikchr.js
 
+#
+# compile_commands.json support...
+#
+# We have to avoid applying compile_commands support to the in-tree
+# tools, as those compile with BCC, which may differ from TCC.
+# e.g. BCC might be gcc (which does not support -MJ ...) while TCC is
+# clang (which does).
+#
+# What follows is more verbose than strictly necessary because we're
+# limited to POSIX make syntax.
+all:
+compile-commands-dir.yes = $(OBJDIR)
+compile-commands-dir.no =
+compile-commands-dir = $(compile-commands-dir.$(MAKE_COMPILATION_DB))
+compile-command-args.yes = -MJ $(TOPDIR)/$(compile-commands-dir)/$(@F:.o=.o.json)
+compile-command-args.no =
+TCCFLAGS += $(compile-command-args.$(MAKE_COMPILATION_DB))
+compile_commands.json = $(TOPDIR)/compile_commands.json
+# compile_commands.json is a concatenation of the .o.json files
+# generated by the compilation process via TCCFLAGS. We have a
+# potential race condition in parallel builds, where a .o.json file is
+# not yet written to completion before compile_commands.json is
+# processed. How to resolve that in a way compatible with POSIX make
+# is unclear.
+#
+# This obscure sed bit ensures that the resulting JSON array does not
+# have a trailing comma.
+$(compile_commands.json): $(OBJ)
+	@-rm -f $@
+	@{ echo '['; cat $(compile-commands-dir)/*.o.json | tr '\n' ' ' | sed -e 's/, $$//'; echo ']'; } > $@
+	@echo "Generated $@"
+compile-commands.no:
+compile-commands.yes: $(compile_commands.json)
+all: compile-commands.$(MAKE_COMPILATION_DB)
+clean: compile-commands-clean
+compile-commands-clean:
+	rm -fr $(compile_commands.json)
+
+#
+# End compile_commands.json support
+#
+
 #
 # The list of all the targets that do not correspond to real files. This stops
 # 'make' from getting confused when someone makes an error in a rule.
 #
 
 .PHONY: all install test clean
-}
+.PHONY: compile-commands-clean compile-commands-dir
+}]
 
 close $output_file
 #
@@ -2051,9 +2094,15 @@ foreach s [lsort $extra_files] {
   set redir {>>}
 }
 
+foreach s [lsort $src] {
+  set extra_h($s) {}
+}
+set extra_h(builtin) " \"\$(OX)\\builtin_data.h\""
+set extra_h(dispatch) " \"\$(OX)\\page_index.h\""
+
 writeln ""
 foreach s [lsort $src] {
-  writeln "\"\$(OX)\\$s\$O\" : \"\$(OX)\\${s}_.c\" \"\$(OX)\\${s}.h\""
+  writeln "\"\$(OX)\\$s\$O\" : \"\$(OX)\\${s}_.c\" \"\$(OX)\\${s}.h\"$extra_h($s)"
   writeln "\t\$(TCC) /Fo\$@ /Fd\$(@D)\\ -c \"\$(OX)\\${s}_.c\"\n"
   writeln "\"\$(OX)\\${s}_.c\" : \"\$(SRCDIR)\\$s.c\""
   writeln "\t\"\$(OBJDIR)\\translate\$E\" \$** > \$@\n"
